#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# Add-on Information                                                                                                                                                                  #
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# This section contains the metadata and information about the add-on, such as its name, version, and author.
# Add this at the top of your script
import bpy
import os
import json
import random
import base64  # Add this line
from bpy.props import EnumProperty, StringProperty, FloatProperty, BoolProperty
from bpy.types import Operator, Panel, PropertyGroup
from mathutils import Vector
from datetime import datetime, timedelta
import uuid  # ➕ Add this if not already
import math

bl_info = {
    "name": "FusionCam Pro",
    "author": "Powered by Fusion Art Studios",
    "version": (2, 1, 0),
    "blender": (4, 0, 0),
    "location": "View3D > Sidebar > FusionCam",
    "description": (
        "Cinematic camera animation toolkit with smart AI suggestions, real-time DOF focus pull, "
        "camera presets, product showcase tools, and timeline controls."
    ),
    "warning": "",
    "doc_url": "https://fusionart.net/fusioncam-pro/",
    "tracker_url": "https://fusionart.net/contact/",
    "category": "Camera Animation",
}

# 🔐 Server URL for license check
LICENSE_VERIFY_URL = "https://fusionart.net/FusionHubAddon/FusionLicenseKeys/Verify_Pro_Key_All_Products.php"

# 📂 Local key file (in Blender config dir)
KEY_FILE = os.path.join(bpy.utils.user_resource('CONFIG'), "fusioncam_key.json")

# 🧠 Product ID for FusionCam (must match PHP)
FUSIONCAM_PRODUCT_ID = "fusion_cam_pro"

# Path to save the key
ADDON_KEY_FILE = os.path.join(bpy.utils.user_resource('SCRIPTS'), "fusioncam_pro_key.json")


DEVICE_ID = str(uuid.getnode())  # Unique device ID
def execute(self, context):
    scene = context.scene
    key = scene.fusioncam_pro_key  # 🛠️ ye line honi chahiye
    device_id = str(uuid.getnode())

    data = {
        "product": "fusion_cam_pro",
        "license_key": key,
        "device_id": device_id
    }

    # Rest of the code...



# Function to save the key to a file

def save_fusioncam_key(key):
    with open(KEY_FILE, "w") as f:
        json.dump({"key": key}, f)

def load_fusioncam_key():
    if os.path.exists(KEY_FILE):
        with open(KEY_FILE, "r") as f:
            return json.load(f).get("key")
    return None

#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# Pro Version Key                                                                                                                                                                     #
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# This is the key required to unlock the Pro version of the add-on.
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# Free and Pro Presets                                                                                                                                                                #
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# These are the presets available in the Free and Pro versions of the add-on.
# Free presets are basic camera animations, while Pro presets offer more advanced and creative options.

# Free Version Presets
FREE_PRESETS = [
    ("Cinematic Wide", "🎥 Cinematic Wide", "Wide-angle shot for cinematic effect"),
    ("Close-Up Drama", "🎬 Close-Up Drama", "Close-up shot with dramatic lighting"),
    ("Tracking Shot", "📹 Tracking Shot", "Smooth tracking movement"),
    ("Crane Shot", "🏗️ Crane Shot", "Elevated shot for dramatic effect"),
    ("Low Angle", "📉 Low Angle", "Low-angle shot for powerful visuals"),
    ("Over-the-Shoulder", "👤 Over-the-Shoulder", "Over-the-shoulder shot for dialogue scenes"),
    ("Dutch Angle", "🎨 Dutch Angle", "Tilted shot for dramatic tension"),
    ("Zoom In", "🔍 Zoom In", "Zoom-in effect for focus"),
    ("Zoom Out", "📤 Zoom Out", "Zoom-out effect for reveal"),
    ("Panning Shot", "🔄 Panning Shot", "Horizontal pan for wide scenes"),
    ("Random Motion", "🎲 Random Motion", "Random motion for creative shots")
]

# Pro Version Presets
PRO_PRESETS = [
    ("Signature Spin", "🌀 Signature Spin", "Elegant 360° spin"),
    ("Velvet Flow", "🪄 Velvet Flow", "Slow-motion dolly-in"),
    ("Opulent Showcase", "💎 Opulent Showcase", "Dramatic side-sweep"),
    ("Infinity Slide", "♾️ Infinity Slide", "Seamless looping pan"),
    ("Gravity Shift", "🌌 Gravity Shift", "Floating effect"),
    ("Prism Rotation", "🌈 Prism Rotation", "Multi-angle rotation"),
    ("Diamond Sweep", "💠 Diamond Sweep", "Precise tracking"),
    ("Eclipse Motion", "🌑 Eclipse Motion", "Half-moon orbit"),
    ("Aurora Glide", "🌠 Aurora Glide", "Diagonal motion with auto-focus"),
    ("Zenith Hover", "🚀 Zenith Hover", "Floating animation with tilt"),
    ("Momentum Reveal", "⚡ Momentum Reveal", "Fast-paced entrance"),
    ("Timeless Elegance", "⌛ Timeless Elegance", "Classic luxury showcase")
]

#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# Cinematic Camera Presets (Free Version)                                                                                                                                             #
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# These are the cinematic camera presets available in the Free version of the add-on.
# Each preset includes camera settings like focal length, aperture, and shutter speed for cinematic effects.

CINEMATIC_PRESETS = [
    ("Static Shot", "📷 Static Shot", "50mm, f/2.8, 1/50s, ISO 100, tripod for stability"),
    ("Orbit Shot", "🔄 Orbit Shot", "35mm, f/2.8, smooth circular motion, slight motion blur"),
    ("Dolly In/Out", "🎥 Dolly In/Out", "50mm, f/2.8, slow ease-in/out, focus on branding"),
    ("Slide Shot", "🛝 Slide Shot", "35mm, f/2.8, steady horizontal movement, smooth transitions"),
    ("Top-Down Shot", "⬇️ Top-Down Shot", "50mm, f/4, flat angle, even soft lighting"),
    ("Macro Shot", "🔬 Macro Shot", "85mm, f/2.8, extreme close-up, sharp focus on textures"),
    ("Hero Shot", "🦸 Hero Shot", "35mm, f/2.8, low angle, dramatic lighting with rim light"),
    ("Slow Motion", "⏳ Slow Motion", "120-240fps, 1/250s, high-speed effects, controlled motion blur"),
    ("Parallax Shot", "🌌 Parallax Shot", "50mm, f/2.8, layered depth, foreground and background movement"),
    ("Wide Angle", "🌄 Wide Angle", "24mm, f/2.8, wide field of view for landscapes"),
    ("Fish Eye", "🐟 Fish Eye", "10mm, f/4, extreme distortion for creative shots"),
    ("Low Key", "🌑 Low Key", "50mm, f/2.8, dark and moody lighting"),
    ("High Key", "☀️ High Key", "50mm, f/4, bright and even lighting"),
    ("Tilt Shift", "🎯 Tilt Shift", "50mm, f/2.8, miniature effect with selective focus"),
    ("Silhouette", "🌇 Silhouette", "35mm, f/2.8, backlit subject with dark outline"),
    ("Golden Hour", "🌅 Golden Hour", "50mm, f/2.8, warm tones during sunset"),
    ("Blue Hour", "🌌 Blue Hour", "50mm, f/2.8, cool tones during twilight"),
    ("Reflection", "🪞 Reflection", "35mm, f/2.8, capture reflections for artistic effect"),
    ("Symmetry", "⚖️ Symmetry", "50mm, f/2.8, perfectly symmetrical composition")
]

#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# New Pro Features                                                                                                                                                                    #
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
#1. draw(self, context): function ke start me yeh logic paste karo:
def draw(self, context):
    layout = self.layout
    scene = context.scene

    if not hasattr(scene, "fusionhub_signed_in") or not hasattr(scene, "fusionhub_license_verified"):
        layout.box().label(text="❌ Fusion Hub not detected!", icon='ERROR')
        return

    if not scene.fusionhub_signed_in or not scene.fusionhub_license_verified:
        box = layout.box()
        box.label(text="🔒 Locked", icon='LOCKED')
        box.label(text="Sign In and Verify License via Fusion Hub", icon='INFO')
        return
#2. (Optional) register() function me safety ke liye:

def update_object_visibility(scene):
    """Hide and disable all objects except the selected ones when enabled."""

    print("Updating object visibility")

    # Check if selective visibility is enabled
    if not scene.selective_object_visibility.enable_selective_visibility:
        print("Selective visibility is disabled")
        # If the feature is disabled, make all objects visible and enabled
        for obj in scene.objects:
            if obj.type == 'MESH':
                obj.hide_set(False)
                obj.hide_viewport = False
        return

    # Get the selected objects
    selected_objects = bpy.context.selected_objects
    if not selected_objects:
        print("No selected objects")
        return

    # Hide and disable all objects except the selected ones
    for obj in scene.objects:
        if obj.type == 'MESH':
            obj.hide_set(obj not in selected_objects)
            obj.hide_viewport = (obj not in selected_objects)

    print("Object visibility updated")



# FUNCTION TO APPLY OFFSET (Step 4)
def apply_keyframe_offset(obj, offset=10):
    if not obj or not obj.animation_data or not obj.animation_data.action:
        print("No animation data found!")
        return False  # ❌ Pehle yeh None return kar raha tha, ab False return karega

    action = obj.animation_data.action
    fcurves = action.fcurves

    for fcurve in fcurves:
        for keyframe in fcurve.keyframe_points:
            keyframe.co.x += offset  # ✅ Keyframes move ho rahe hain

    bpy.context.scene.frame_set(bpy.context.scene.frame_current)
    print(f"Keyframes moved forward by {offset} frames")
    return True  # ✅ Ensure karo ke kuch return ho raha ho

# Property for UI slider
bpy.types.Scene.keyframe_offset_amount = bpy.props.IntProperty(
    name="Offset Amount",
    description="Amount of frames to offset keyframes",
    default=10,
    min=-100,
    max=100
)

def apply_easing_to_all_keyframes(obj, easing_type, intensity=1.0):
    """Apply easing to all keyframes of an object."""
    if not obj or not obj.animation_data or not obj.animation_data.action:
        return  # Safe exit if no animation data or action is found

    for fcurve in obj.animation_data.action.fcurves:
        for keyframe in fcurve.keyframe_points:
            apply_easing(keyframe, easing_type, intensity)


bpy.types.Scene.Easing_Pro_intensity = bpy.props.FloatProperty(
    name="Easing Intensity",
    description="Control the intensity of easing",
    default=1.0,
    min=0.0,
    max=10.0
)

# Example usage:

def apply_easing(keyframe, easing_type, intensity=1.0):
    """Apply easing to a keyframe based on the easing type."""
    if easing_type == "LINEAR":
        keyframe.interpolation = 'LINEAR'

    elif easing_type == "BEZIER":
        keyframe.interpolation = 'BEZIER'
        keyframe.handle_left_type = 'AUTO'
        keyframe.handle_right_type = 'AUTO'

    elif easing_type == "EXPONENTIAL":
        keyframe.interpolation = 'BACK'

    elif easing_type == "BOUNCE":
        keyframe.interpolation = 'BOUNCE'
        keyframe.amplitude = intensity  # Control the intensity of the effect

    elif easing_type == "ELASTIC":
        keyframe.interpolation = 'ELASTIC'
        keyframe.amplitude = intensity  # Control the intensity of the effect

    elif easing_type == "SINE":
        keyframe.interpolation = 'SINE'

    elif easing_type == "QUADRATIC":
        keyframe.interpolation = 'QUAD'

    elif easing_type == "CUBIC":
        keyframe.interpolation = 'CUBIC'

    elif easing_type == "QUARTIC":
        keyframe.interpolation = 'QUART'

    elif easing_type == "QUINTIC":
        keyframe.interpolation = 'QUINT'

    elif easing_type == "EASE_IN":
        keyframe.interpolation = 'BEZIER'
        keyframe.handle_left_type = 'VECTOR'

    elif easing_type == "EASE_OUT":
        keyframe.interpolation = 'BEZIER'
        keyframe.handle_left_type = 'AUTO'
        keyframe.handle_right_type = 'AUTO'

    elif easing_type == "AUTO_SMOOTH":
        keyframe.interpolation = 'BEZIER'
        keyframe.handle_left_type = 'AUTO'
        keyframe.handle_right_type = 'AUTO'

    elif easing_type == "AUTO_LINEAR":
        keyframe.interpolation = 'LINEAR'
        keyframe.handle_right_type = 'VECTOR'

def apply_settings(cam):
    """
    Apply camera settings like easing, keyframe density, and randomization.
    """
    if not cam or not cam.animation_data:
        return

    # Get settings from the scene
    easing = bpy.context.scene.Easing_Pro
    density = bpy.context.scene.fusioncam_density
    randomize = bpy.context.scene.fusioncam_random  # Yeh line add karein

    # Adjust keyframe interpolation and timing
    for fcurve in cam.animation_data.action.fcurves:
        for keyframe in fcurve.keyframe_points:
            if randomize:  # Ab randomize sahi se define hai
                keyframe.co[1] += random.uniform(-0.1, 0.1)
                keyframe.handle_left[1] += random.uniform(-0.1, 0.1)
                keyframe.handle_right[1] += random.uniform(-0.1, 0.1)

class MakeCameraUnselectableOperator(bpy.types.Operator):
    """Hide_select"""
    bl_idname = "camera.make_unselectable"
    bl_label = "Hide_select"

    def execute(self, context):
        cam = context.scene.camera
        if cam:
            cam.hide_select = not cam.hide_select  # Toggle selection lock
            state = "Unselectable" if cam.hide_select else "Selectable"
            self.report({'INFO'}, f"Camera is now {state}")
        else:
            self.report({'WARNING'}, "No active camera found!")
        return {'FINISHED'}

def apply_settings(cam):
    if cam:
        cam.data.lens = 50  # Example setting
        cam.data.clip_start = 0.1
        cam.data.clip_end = 1000
        print("Camera settings applied!")


class FIT_OT_CameraToProduct(bpy.types.Operator):
    """Adjusts Camera to Frame the Product Object"""
    bl_idname = "camera.fit_to_frame"
    bl_label = "Fit Camera to Frame"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        product_layer = None

        # Find the object with "Product" in its name
        for obj in bpy.data.objects:
            if "Product" in obj.name:
                product_layer = obj
                break

        if product_layer:
            camera = bpy.context.scene.camera
            if camera:
                # Move the camera to frame the product
                bpy.context.view_layer.objects.active = camera
                camera.location = (product_layer.location.x, product_layer.location.y - 3, product_layer.location.z + 2)
                camera.rotation_euler = (1.2, 0, 0)  # Adjust rotation for a better view

                # Set camera to track the object
                constraint = camera.constraints.new(type='TRACK_TO')
                constraint.target = product_layer
                constraint.track_axis = 'TRACK_NEGATIVE_Z'
                constraint.up_axis = 'UP_Y'

                bpy.context.view_layer.update()
                self.report({'INFO'}, "Camera adjusted to fit product in frame!")
            else:
                self.report({'ERROR'}, "No camera found in the scene!")
        else:
            self.report({'ERROR'}, "No product layer found!")

        return {'FINISHED'}

def bind_camera_to_object(camera, target_object):
    """
    Bind the camera to a target object using a Track To constraint.
    """
    if not camera or not target_object:
        return {'CANCELLED'}

    # Add a Track To constraint to the camera
    track_to_constraint = camera.constraints.new(type='TRACK_TO')
    track_to_constraint.target = target_object
    track_to_constraint.track_axis = 'TRACK_NEGATIVE_Z'  # Camera looks at the object
    track_to_constraint.up_axis = 'UP_Y'  # Up axis for the camera

    return {'FINISHED'}


def add_camera_shake(camera, intensity=0.1, duration=50):
    """Add a camera shake effect."""
    for frame in range(bpy.context.scene.frame_start, bpy.context.scene.frame_start + duration):
        camera.location.x += random.uniform(-intensity, intensity)
        camera.location.y += random.uniform(-intensity, intensity)
        camera.keyframe_insert(data_path="location", frame=frame)

def animate_camera_along_path(camera, curve):
    """Animate the camera along a custom path."""
    follow_path = camera.constraints.new(type='FOLLOW_PATH')
    follow_path.target = curve
    follow_path.use_curve_follow = True

def real_time_preview(camera):
    """Enable real-time preview of the camera animation."""
    bpy.context.scene.camera = camera
    bpy.context.space_data.region_3d.view_perspective = 'CAMERA'


def ai_suggest_camera_angle(camera, target_obj, total_shots=25, radius=5):
    """AI randomly sets camera position around the product (no animation)."""

    # Randomly select an angle
    i = random.randint(0, total_shots - 1)  # Random shot pick
    angle = (i / total_shots) * 2 * math.pi  # 0 to 2π radians

    # Calculate camera position around the target
    cam_x = target_obj.location.x + radius * math.cos(angle)
    cam_y = target_obj.location.y + radius * math.sin(angle)
    cam_z = target_obj.location.z + 2  # Thoda upar se

    camera.location = (cam_x, cam_y, cam_z)

    # Rotate camera to look at the target
    direction = target_obj.location - camera.location
    rot_quat = direction.to_track_quat('-Z', 'Y')
    camera.rotation_euler = rot_quat.to_euler()

    # Set as active camera (optional)
    bpy.context.scene.camera = camera

    print(f"📸 Camera positioned at random angle {round(math.degrees(angle))}° around the product.")


def lock_camera_to_viewport(camera):
    """Lock the camera to the current viewport perspective."""
    bpy.ops.view3d.camera_to_view()

def fit_to_frame(camera, target_object):
    """Fit the selected object in the camera's view."""
    bpy.ops.view3d.camera_to_view_selected()
    auto_frame_object(camera, target_object)

def create_camera_and_control(preset, is_cinematic=False):
    """
    Create a camera, empty object, and bone for the given preset.
    If is_cinematic is True, the camera will be created with cinematic settings.
    """
    # Create an empty object (control object)
    empty_name = f"Control_{preset}_Cinematic" if is_cinematic else f"Control_{preset}"
    empty = bpy.data.objects.new(empty_name, None)
    empty.empty_display_type = 'ARROWS'  # Professional look
    bpy.context.scene.collection.objects.link(empty)

    # Create a new camera
    cam_name = f"Camera_{preset}_Cinematic" if is_cinematic else f"Camera_{preset}"
    cam_data = bpy.data.cameras.new(cam_name)
    camera = bpy.data.objects.new(cam_name, cam_data)
    bpy.context.scene.collection.objects.link(camera)

    # Parent the camera to the empty object
    camera.parent = empty

    # Rotate the camera 90 degrees on the X-axis
    camera.rotation_euler = (1.5708, 0, 0)  # 90 degrees in radians

    # Set the new camera as the active camera
    bpy.context.scene.camera = camera

    return camera, empty

def apply_camera_animation(preset):
    """
    Apply camera animation based on the selected preset.
    """
    # Create a new camera and control object for this animation
    camera, control = create_camera_and_control(preset)

    # Point the camera towards the product (origin)
    track_to = camera.constraints.new(type='TRACK_TO')
    product = bpy.context.scene.objects.get("Product")
    if product:
        track_to.target = product
    else:
        return "Product object not found!"

    track_to.track_axis = 'TRACK_NEGATIVE_Z'
    track_to.up_axis = 'UP_Y'

    frame_start = bpy.context.scene.frame_start
    frame_end = bpy.context.scene.frame_end


#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# Camera Creation and Animation Functions                                                                                                                                             #
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# This section contains functions for creating cameras, applying animations, and managing camera settings.

    # Apply presets with animation
    if preset == "Cinematic Wide":
        control.location = (0, -5, 2)
        control.keyframe_insert(data_path="location", frame=frame_start)
        control.location = (0, 5, 2)
        control.keyframe_insert(data_path="location", frame=frame_end)
    elif preset == "Close-Up Drama":
        control.location = (0, -1, 1)
        control.keyframe_insert(data_path="location", frame=frame_start)
        control.location = (0, 1, 1)
        control.keyframe_insert(data_path="location", frame=frame_end)
    elif preset == "Tracking Shot":
        control.location = (-5, -5, 2)
        control.keyframe_insert(data_path="location", frame=frame_start)
        control.location = (5, 5, 2)
        control.keyframe_insert(data_path="location", frame=frame_end)
    elif preset == "Crane Shot":
        control.location = (0, -2, 1)
        control.keyframe_insert(data_path="location", frame=frame_start)
        control.location = (0, -2, 5)
        control.keyframe_insert(data_path="location", frame=frame_end)
    elif preset == "Low Angle":
        control.location = (0, -2, 0.5)
        control.keyframe_insert(data_path="location", frame=frame_start)
        control.location = (0, 2, 0.5)
        control.keyframe_insert(data_path="location", frame=frame_end)
    elif preset == "Over-the-Shoulder":
        control.location = (-1, -2, 1.5)
        control.keyframe_insert(data_path="location", frame=frame_start)
        control.location = (1, -2, 1.5)
        control.keyframe_insert(data_path="location", frame=frame_end)
    elif preset == "Dutch Angle":
        control.rotation_euler = (6.5708, 0, 0.5)  # Tilted angle
        control.keyframe_insert(data_path="rotation_euler", frame=frame_start)
        control.rotation_euler = (4.5708, 0, -0.5)
        control.keyframe_insert(data_path="rotation_euler", frame=frame_end)
    elif preset == "Zoom In":
        # Simulate zoom by moving the camera closer
        control.location = (0, -5, 2)
        control.keyframe_insert(data_path="location", frame=frame_start)
        control.location = (0, -2, 2)
        control.keyframe_insert(data_path="location", frame=frame_end)
    elif preset == "Zoom Out":
        # Simulate zoom by moving the camera farther
        control.location = (0, -2, 2)
        control.keyframe_insert(data_path="location", frame=frame_start)
        control.location = (0, -5, 2)
        control.keyframe_insert(data_path="location", frame=frame_end)
    elif preset == "Panning Shot":
        control.location = (-5, 0, 2)
        control.keyframe_insert(data_path="location", frame=frame_start)
        control.location = (5, 0, 2)
        control.keyframe_insert(data_path="location", frame=frame_end)
    elif preset == "Random Motion":
        control.location = (random.uniform(-5, 5), random.uniform(-5, 5), random.uniform(1, 5))
        control.keyframe_insert(data_path="location", frame=frame_start)
        control.location = (random.uniform(-5, 5), random.uniform(-5, 5), random.uniform(1, 5))
        control.keyframe_insert(data_path="location", frame=frame_end)
    elif preset == "Signature Spin":
        control.rotation_euler = (0, 1, 4)
        control.keyframe_insert(data_path="rotation_euler", frame=frame_start)
        control.rotation_euler = (0, 0, 6.28319)  # 360 degrees in radians
        control.keyframe_insert(data_path="rotation_euler", frame=frame_end)
    elif preset == "Velvet Flow":
        control.location = (0, -5, 2)
        control.keyframe_insert(data_path="location", frame=frame_start)
        control.location = (0, 0, 2)
        control.keyframe_insert(data_path="location", frame=frame_end)
    elif preset == "Opulent Showcase":
        control.location = (-5, -5, 2)
        control.keyframe_insert(data_path="location", frame=frame_start)
        control.location = (5, 5, 2)
        control.keyframe_insert(data_path="location", frame=frame_end)
    elif preset == "Infinity Slide":
        control.location = (-5, 0, 2)
        control.keyframe_insert(data_path="location", frame=frame_start)
        control.location = (5, 0, 2)
        control.keyframe_insert(data_path="location", frame=frame_end)
    elif preset == "Gravity Shift":
        control.location = (0, 0, 2)
        control.keyframe_insert(data_path="location", frame=frame_start)
        control.location = (0, 0, 5)
        control.keyframe_insert(data_path="location", frame=frame_end)
    elif preset == "Prism Rotation":
        control.rotation_euler = (0, 0, 0)
        control.keyframe_insert(data_path="rotation_euler", frame=frame_start)
        control.rotation_euler = (1.5708, 1.5708, 1.5708)  # Multi-angle rotation
        control.keyframe_insert(data_path="rotation_euler", frame=frame_end)
    elif preset == "Diamond Sweep":
        control.location = (-5, -5, 2)
        control.keyframe_insert(data_path="location", frame=frame_start)
        control.location = (5, 5, 2)
        control.keyframe_insert(data_path="location", frame=frame_end)
    elif preset == "Eclipse Motion":
        control.rotation_euler = (0, 0, 0)
        control.keyframe_insert(data_path="rotation_euler", frame=frame_start)
        control.rotation_euler = (0, 0, 3.14159)  # 180 degrees in radians
        control.keyframe_insert(data_path="rotation_euler", frame=frame_end)
    elif preset == "Aurora Glide":
        control.location = (-5, -5, 2)
        control.keyframe_insert(data_path="location", frame=frame_start)
        control.location = (5, 5, 2)
        control.keyframe_insert(data_path="location", frame=frame_end)
    elif preset == "Zenith Hover":
        control.location = (0, 0, 2)
        control.keyframe_insert(data_path="location", frame=frame_start)
        control.location = (0, 0, 5)
        control.keyframe_insert(data_path="location", frame=frame_end)
    elif preset == "Momentum Reveal":
        control.location = (-5, -5, 2)
        control.keyframe_insert(data_path="location", frame=frame_start)
        control.location = (5, 5, 2)
        control.keyframe_insert(data_path="location", frame=frame_end)
    elif preset == "Timeless Elegance":
        control.location = (0, -5, 2)
        control.keyframe_insert(data_path="location", frame=frame_start)
        control.location = (0, 5, 2)
        control.keyframe_insert(data_path="location", frame=frame_end)

    # Apply camera settings
    apply_settings(camera)

    print(f"Applied camera preset: {preset}")
    bpy.ops.screen.animation_play()

def add_cinematic_camera(preset):
    """
    Add a cinematic camera with the selected preset settings.
    This function does not apply any animation.
    """
    # Create a new camera and control object
    camera, control = create_camera_and_control(preset, is_cinematic=True)

    # Apply cinematic settings based on the preset
    if preset == "Static Shot":
        camera.data.lens = 50  # 50mm focal length
        camera.data.dof.focus_distance = 2.0  # Focus distance
    elif preset == "Orbit Shot":
        camera.data.lens = 35  # 35mm focal length
        camera.data.dof.focus_distance = 1.5  # Focus distance
    elif preset == "Dolly In/Out":
        camera.data.lens = 55  # 50mm focal length
        camera.data.dof.focus_distance = 2.5  # Focus distance
    elif preset == "Slide Shot":
        camera.data.lens = 33  # 35mm focal length
        camera.data.dof.focus_distance = 2.0  # Focus distance
    elif preset == "Top-Down Shot":
        camera.data.lens = 54  # 50mm focal length
        camera.data.dof.focus_distance = 3.0  # Focus distance
    elif preset == "Macro Shot":
        camera.data.lens = 85  # 85mm focal length
        camera.data.dof.focus_distance = 0.5  # Focus distance
    elif preset == "Hero Shot":
        camera.data.lens = 25  # 35mm focal length
        camera.data.dof.focus_distance = 2.0  # Focus distance
    elif preset == "Slow Motion":
        camera.data.lens = 64  # 50mm focal length
        camera.data.dof.focus_distance = 2.5  # Focus distance
    elif preset == "Parallax Shot":
        camera.data.lens = 35  # 35mm focal length
        camera.data.dof.focus_distance = 3.0  # Focus distance
    elif preset == "Fish Eye":
        camera.data.lens = 28  # 28mm focal length
        camera.data.dof.focus_distance = 3.0  # Focus distance
    elif preset == "Wide Angle":
        camera.data.lens = 10  # 10mm focal length
        camera.data.dof.focus_distance = 3.0  # Focus distance
    elif preset == "Low Key":
        camera.data.lens = 48  # 48mm focal length
        camera.data.dof.focus_distance = 3.0  # Focus distance
    elif preset == "High Key":
        camera.data.lens = 25  # 55mm focal length
        camera.data.dof.focus_distance = 3.0  # Focus distance
    elif preset == "Tilt Shift":
        camera.data.lens = 33  # 33mm focal length
        camera.data.dof.focus_distance = 3.0  # Focus distance
    elif preset == "Silhouette":
        camera.data.lens = 30  # 30mm focal length
        camera.data.dof.focus_distance = 3.0  # Focus distance
    elif preset == "Golden Hour":
        camera.data.lens = 56  # 56mm focal length
        camera.data.dof.focus_distance = 3.0  # Focus distance
    elif preset == "Blue Hour":
        camera.data.lens = 32  # 30mm focal length
        camera.data.dof.focus_distance = 3.0  # Focus distance
    elif preset == "Reflection":
        camera.data.lens = 26  # 26mm focal length
        camera.data.dof.focus_distance = 3.0  # Focus distance
    elif preset == "Symmetry":
        camera.data.lens = 22  # 22mm focal length
        camera.data.dof.focus_distance = 3.0  # Focus distance

    print(f"Added cinematic camera: {preset}")

def draw_func(self, context):
    layout = self.layout
    layout.operator("view3d.lock_camera_view")


def set_timeline_to_100(camera, start_frame, end_frame):
    """Set_Timeline_To_100 between start and end frames."""
    if not camera:
        return {'CANCELLED'}

    # Set animation loop
    bpy.context.scene.frame_start = start_frame
    bpy.context.scene.frame_end = end_frame
    print("Animation loop set!")
    return {'FINISHED'}

def execute(self, context):
    scene = context.scene
    camera = scene.camera  # Active Camera

    print(f"DEBUG: Current frame value = {self.frame}")  # Debugging line

    if not camera:
        self.report({'WARNING'}, "No active camera found!")
        return {'CANCELLED'}

    marker = scene.timeline_markers.get(f"Marker_{self.frame}")
    if not marker:
        marker = scene.timeline_markers.new(name=f"Marker_{self.frame}", frame=self.frame)

    marker.camera = camera
    self.report({'INFO'}, f"Camera bound to frame {self.frame}")
    return {'FINISHED'}

# 📌 Function to Toggle DOF
def toggle_dof(enable_dof):
    """Enable or Disable DOF for the active camera."""
    scene = bpy.context.scene
    camera = scene.camera

    if camera:
        camera.data.dof.use_dof = enable_dof
        print(f"🔄 DOF {'ENABLED' if enable_dof else 'DISABLED'} for Camera: {camera.name}")
    else:
        print("⚠ No active camera found!")

# This section defines the UI panel for the add-on, including buttons, dropdowns, and other UI elements.
class FUSIONCAM_OT_ApplyEasing(bpy.types.Operator):
    bl_idname = "fusioncam.apply_easing"
    bl_label = "Apply Easing"

    def execute(self, context):
        scene = context.scene
        easing_type = scene.Easing_Pro
        intensity = scene.Easing_Pro_intensity

        # Apply easing to all objects with keyframes
        for obj in bpy.context.scene.objects:
            if obj.animation_data:
                apply_easing_to_all_keyframes(obj, easing_type, intensity)

        self.report({'INFO'}, f"Applied {easing_type} easing to all keyframes!")
        return {'FINISHED'}



#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
#AL NEW FEATURES WILL BE ADDED HERE FOR FUTURE                                                                                                                                                                      #
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#


class FUSIONCAM_OT_RecommendCameraPreset(bpy.types.Operator):
    bl_idname = "fusioncam.recommend_camera"
    bl_label = "🧠 Smart Recommend Preset"

    def execute(self, context):
        objects = context.scene.objects
        product_like = [obj for obj in objects if "product" in obj.name.lower()]
        character_like = [obj for obj in objects if "character" in obj.name.lower()]

        if product_like:
            context.scene.fusioncam_preset = "Opulent Showcase"
            self.report({'INFO'}, "Recommended: Opulent Showcase (Product Detected)")
        elif character_like:
            context.scene.fusioncam_preset = "Close-Up Drama"
            self.report({'INFO'}, "Recommended: Close-Up Drama (Character Detected)")
        else:
            context.scene.fusioncam_preset = "Cinematic Wide"
            self.report({'INFO'}, "Recommended: Cinematic Wide (Default Fallback)")
        return {'FINISHED'}


class FUSIONCAM_OT_LockProVersion(bpy.types.Operator):
    bl_idname = "fusioncam.lock_pro"
    bl_label = "🔒 Lock Pro Version"
    bl_description = "Lock the Pro version for development testing"

    def execute(self, context):
        # Delete the saved key file
        if os.path.exists(ADDON_KEY_FILE):
            os.remove(ADDON_KEY_FILE)

        # Reset the unlock status
        context.scene.fusioncam_pro_unlocked = False
        context.scene.fusioncam_pro_key = ""

        self.report({'INFO'}, "Pro version locked for testing.")
        return {'FINISHED'}


def update_dynamic_pro_tip(scene):
    camera = scene.camera

    if not camera:
        scene.fusioncam_dynamic_tip = "🎥 Add a Camera to start your cinematic shot."
        return

    cam_data = camera.data

    # --- Start Smart Priority Checks ---

    # 1. Focus Object not set
    if not cam_data.dof.focus_object:
        scene.fusioncam_dynamic_tip = "🎯 Set Focus on your product for accurate Depth of Field."
        return

    # 2. Camera not renamed (still default name)
    if camera.name.lower() in ("camera", "camera.001", "camera.002", "camera.003"):
        scene.fusioncam_dynamic_tip = "✏️ Rename Camera for better scene organization."
        return

    # 3. DOF on but Focus Distance zero (limit not adjusted)
    if cam_data.dof.use_dof and (cam_data.dof.focus_distance == 0):
        scene.fusioncam_dynamic_tip = "🎯 Adjust Focus Distance to control Depth of Field."
        return

    # 4. Camera not bound to timeline (no keyframes)
    if not camera.animation_data or not camera.animation_data.action:
        scene.fusioncam_dynamic_tip = "🎬 Bind Camera to Timeline for keyframed motion."
        return

    # 5. Camera not bound to object (Track To Constraint missing)
    if not any(c.type == 'TRACK_TO' for c in camera.constraints):
        scene.fusioncam_dynamic_tip = "🎯 Fit To Frame Your Camera To Steady Shoot."
        return

    # 6. FusionCam RealTime Preview OFF
    if hasattr(scene, "fusioncam_realtime_preview") and not scene.fusioncam_realtime_preview:
        scene.fusioncam_dynamic_tip = "👁️ Enable Realtime Preview for live cinematic feedback."
        return

    # 7. Depth of Field not enabled
    if not cam_data.dof.use_dof:
        scene.fusioncam_dynamic_tip = "🎯 Enable Depth of Field for cinematic shots."
        return

    # 8. Lens is still default 50mm
    if cam_data.lens == 50:
        scene.fusioncam_dynamic_tip = "📸 Try 35mm or 85mm lens for dramatic compositions."
        return

    # 9. Camera rotation static (no angle for dynamic motion)
    if camera.rotation_euler.length == 0:
        scene.fusioncam_dynamic_tip = "🎥 Add slight rotation for dynamic camera movement."
        return

    # 10. Motion Blur is OFF
    if not scene.render.use_motion_blur:
        scene.fusioncam_dynamic_tip = "🌀 Enable Motion Blur for natural movement."
        return

    # 11. Motion Blur shutter speed incorrect
    if hasattr(scene, "render") and (scene.render.motion_blur_shutter < 0.1 or scene.render.motion_blur_shutter > 1.5):
        scene.fusioncam_dynamic_tip = "⚡ Adjust Motion Shutter Speed for smoothness."
        return

    # 12. Render Samples too low
    if hasattr(scene, "cycles") and scene.cycles.samples < 32:
        scene.fusioncam_dynamic_tip = "📷 Increase Render Samples for cleaner final renders."
        return

    # 13. Safe Frames not enabled
    if not scene.render.use_border:
        scene.fusioncam_dynamic_tip = "🎞️ Enable Safe Frames for proper composition framing."
        return

    # 14. Camera Clip Start too far (0.5+)
    if cam_data.clip_start > 0.5:
        scene.fusioncam_dynamic_tip = "🔍 Reduce Clip Start for better close-up shots."
        return

    # 15. Filmic Color Management not active
    if scene.view_settings.view_transform != 'Filmic':
        scene.fusioncam_dynamic_tip = "🎨 Switch to Filmic Color Management for cinematic grading."
        return

    # --- All Good ---
    scene.fusioncam_dynamic_tip = "✅ Perfect Setup! Ready for Cinematic Render."

# Properties for Smart Pro Tip
def fusioncam_define_tip_properties():
    bpy.types.Scene.fusioncam_dynamic_tip = bpy.props.StringProperty(
        name="FusionCam Dynamic Tip",
        description="Smart real-time tips based on scene and camera setup",
        default=""
    )

def check_camera_presence(scene):
    scene.fusioncam_show_camera_warning = not any(obj.type == 'CAMERA' for obj in scene.objects)

#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# UI Panel                                                                                                                                                                            #
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#

class FUSIONCAM_PT_Panel(bpy.types.Panel):
    bl_label = "FusionCam PRO V2"
    bl_idname = "VIEW3D_PT_fusioncam_v2"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'FusionCam Pro'

    def draw(self, context):
        layout = self.layout
        scene = context.scene  # Add this line


        # 🔐 LOCK CHECK - Fixed version
        if not hasattr(context.scene, "fusionhub_logged_in") or not hasattr(context.scene, "fusionhub_license_verified"):
            box = layout.box()
            box.alert = True
            box.label(text="❌ Fusion Hub is not installed!", icon='ERROR')
            box.label(text="Please install Fusion Hub to use FusionCam Pro", icon='INFO')

            row = box.row()
            row.operator("wm.url_open", text="Download Fusion Hub", icon='IMPORT').url = "https://fusionart.net/fusion_hub/"
            return

        if not scene.fusionhub_logged_in or not scene.fusionhub_license_verified:
            box = layout.box()
            box.alert = True
            box.scale_y = 0.9

            row = box.row(align=True)
            row.label(text="", icon='ERROR')
            row.label(text="Fusion Hub Integration Required")

            # --- Start inner box ---
            inner_box = box.box()
            inner_box.label(text="This feature requires the Fusion Hub add-on", icon='ERROR')
            inner_box.label(text="Please install and enable it to continue", icon='BLANK1')
            # --- End inner box ---

            row = box.row()
            row.scale_y = 1.4
            op = row.operator("wm.url_open", text="Download Fusion Hub", icon='IMPORT')
            op.url = "https://fusionart.net/fusion_hub/"

            # Main container
            main_box = layout.box()
            main_box.alert = True
            main_box.scale_y = 0.9

            # Header
            header = main_box.row(align=True)
            header.label(text="", icon='LOCKED')
            header.label(text="Premium Feature Locked")

            # Status panel
            status_box = main_box.box()
            split = status_box.split(factor=0.35, align=True)

            # Status labels
            labels = split.column(align=True)
            labels.scale_x = 1.2
            labels.alignment = 'RIGHT'
            labels.label(text="Account:")
            labels.label(text="License:")

            # Status values
            values = split.column(align=True)
            account_status = values.row(align=True)
            account_status.label(text="Not Signed In" if not scene.fusionhub_logged_in else "Active")
            account_status.label(icon='X' if not scene.fusionhub_logged_in else 'CHECKMARK')

            license_status = values.row(align=True)
            license_status.label(text="Invalid" if not scene.fusionhub_license_verified else "Valid")
            license_status.label(icon='X' if not scene.fusionhub_license_verified else 'CHECKMARK')

            # Action buttons
            main_box.separator(factor=0.8)
            actions = main_box.row(align=True)
            actions.scale_y = 1.3

            login = actions.operator("wm.url_open", text="Visit Our Store", icon='USER')
            login.url = "https://fusionart.net/fusion-shop/"

            purchase = actions.operator("wm.url_open", text="Get Fusion Cam V2.0 License", icon='SOLO_ON')
            purchase.url = "https://fusionart.net/product/fusion-cam/"

            # Help section
            main_box.separator(factor=0.8)
            help_box = main_box.box()
            help_box.label(text="Need Help?", icon='QUESTION')

            help_col = help_box.column(align=True)
            help_col.scale_y = 1.2

            docs = help_col.operator("wm.url_open", text="Documentation", icon='HELP')
            docs.url = "https://fusionart.net/fusion-plugins/"

            support = help_col.operator("wm.url_open", text="Contact Support", icon='URL')
            support.url = "mailto:fusionart3.0@gmail.com"

            return



        # 🔓 IF PASSED, draw full UI below

        if scene.fusioncam_show_camera_warning:
            # Warning: No Camera
            box = layout.box()
            box.label(text="⚠️ Add a Camera to Unlock Pro Settings", icon='INFO')
        else:
            # Smart Pro Tip after Camera Added
            box = layout.box()
            box.label(text="💡 Smart Pro Tip", icon='LIGHT')
            box.label(text=scene.fusioncam_dynamic_tip, icon='INFO')

        # 🔒 Developer control with left text and right-aligned button
        row = layout.row()

        # Left side text
        left_col = row.row()
        left_col.alignment = 'LEFT'
        left_col.label(text="Welcome To Fusion Art Studios:")

        # Right side button
        right_col = row.row()
        right_col.alignment = 'RIGHT'

        op = right_col.operator("fusioncam.lock_pro",
                            text="🔐 LOGOUT",
                            icon='UNLOCKED')

        # Create tabs for Free and Pro versions
        row = layout.row()
        row.prop(context.scene, "fusioncam_tabs", expand=True)
        if context.scene.fusioncam_tabs == "FREE":
            self.draw_free_version(layout, context)
        elif context.scene.fusioncam_tabs == "PRO":
            self.draw_pro_version(layout, context)

        layout = self.layout

        # 📦 Offset Animation Layers
        box = layout.box()
        box.label(text="📦 Offset Animation Layers", icon='ANIM')
        row = box.row(align=True)
        row.prop(context.scene, "keyframe_offset_amount", text="Offset Amount")
        row.operator("object.apply_keyframe_offset", text="Apply Offset", icon='MOD_TIME')

        # 🎚️ Easing Effects Section
        box = layout.box()
        box.label(text="🎚️ Easing Effects", icon='IPO_EASE_IN_OUT')
        row = box.row(align=True)
        row.prop(context.scene, "Easing_Pro", text="Easing Type")
        row = box.row(align=True)
        row.prop(context.scene, "Easing_Pro_intensity", text="Easing Intensity")
        box.operator("fusioncam.apply_easing", text="Apply Easing", icon='ACTION_TWEAK')

#==============================================================================#
        layout.separator()

        # 🚀 Fusion Hub Highlight Section
        hub_box = layout.box()
        hub_box.label(text="LATEST FROM FUSION", icon='FUND')

        row = hub_box.row(align=True)
        row.label(text="👉 Try our all-in-one plugin hub:")
        row.operator("wm.url_open", text="🛒  GET FUSION HUB").url = "https://fusionart.net/fusion_hub/"

        # 💬 Help & Support
        support_box = layout.box()
        support_box.label(text="💬 Help & Support Center", icon='INFO')
        row = support_box.row(align=True)
        row.operator("wm.url_open", text="📞 Contact Us").url = "https://fusionart.net/contact/"
        row.operator("wm.url_open", text="💡 Feature Request").url = "https://fusionart.net/fusion-plugins/"
        row = support_box.row(align=True)
        row.operator("wm.url_open", text="🛒 Store").url = "https://fusionart.net/fusion-shop/"
        row.operator("wm.url_open", text="🌐 Join Discord").url = "https://discord.gg/MtCPMc89pE"

        # 💼 Branding Footer
        branding_box = layout.box()
        branding_box.label(text="⚡ POWERED BY FUSION ART",)

        row = branding_box.row(align=True)
        row.operator("wm.url_open", text="🌐  Visit Fusion Website").url = "https://fusionart.net/fusion-plugins/"
#==============================================================================#
    def draw_free_version(self, layout, context):
        """Draw UI elements for the Free version."""
        layout.label(text="🎬 Free Version Presets:")
        layout.prop(context.scene, "fusioncam_preset_free", text="")
        layout.operator("fusioncam.apply_camera", text="🚀 Apply Camera Preset").preset_type = "FREE"



    def draw_pro_version(self, layout, context):
        scene = context.scene
        camera = scene.camera
        is_pro_unlocked = scene.fusioncam_pro_unlocked

        if not is_pro_unlocked:
            layout.label(text="🔒 Pro Version is Locked", icon='LOCKED')

            box = layout.box()
            box.label(text="Enter your Pro Key to unlock features:")
            box.prop(scene, "fusioncam_pro_key", text="🔑 Pro Key")

            row = box.row()
            row.scale_y = 1.2
            row.operator("fusioncam.validate_key", text="🆙 Upgrade to Pro", icon='KEY_HLT')

            layout.separator()

        # Pro Presets
        layout.label(text="🎬 Pro Version Presets:")
        row = layout.row()
        row.prop(scene, "fusioncam_preset", text="")
        row.enabled = is_pro_unlocked
        row = layout.row()
        row.operator("fusioncam.apply_camera", text="🚀 Apply Camera Preset").preset_type = "PRO"
        row.enabled = is_pro_unlocked
        row = layout.row()
        row.operator("fusioncam.next_preset", text="⏭️ Next Preset")
        row.operator("fusioncam.previous_preset", text="⏮️ Previous Preset")
        row.enabled = is_pro_unlocked


        # 🎥 Cinematic Section
        layout.separator()
        box = layout.box()
        box.label(text="🎬 Cinematic Camera", icon='CAMERA_DATA')
        box.prop(scene, "fusioncam_cinematic_preset", text="Preset")
        box.operator("fusioncam.add_cinematic_camera", text="🎥 Add Cinematic Camera")


        if camera:
            box.enabled = is_pro_unlocked
            # ⚙️ Pro Settings
            box = layout.box()
            row = box.row(align=True)
            row.label(text="🤖 AI-Powered Camera")
            row.enabled = context.scene.fusioncam_pro_unlocked  # 🔒 Only lock the button
            row.operator("fusioncam.ai_camera", text="Activate", icon='MOD_SIMPLIFY')


            box = layout.box()
            # Create a row with space between label and toggle
            row = box.row(align=True)
            row.alignment = 'EXPAND'  # This will push elements to edges

            # Left-aligned label with better wording
            left_col = row.row(align=True)
            left_col.alignment = 'LEFT'
            left_col.label(text="Enable to Show Only the Selected Object", icon='HIDE_OFF')  # More professional wording

            # Right-aligned toggle with ON/OFF state
            right_col = row.row(align=True)
            right_col.alignment = 'RIGHT'
            right_col.scale_x = 1.5  # ⬅️ Increase button width
            right_col.scale_y = 1.2  # ⬅️ Optional: gives some height weight

            right_col.prop(
                scene.selective_object_visibility,
                "enable_selective_visibility",
                text="TURN ON" if scene.selective_object_visibility.enable_selective_visibility else "TURN OFF",
                toggle=True,
                icon='HIDE_OFF' if scene.selective_object_visibility.enable_selective_visibility else 'HIDE_ON'
            )

            box = layout.box()
            box.label(text="⚙️ Pro Settings:")
            row = box.row()
            row.prop(scene, "frame_start", text="⏩ Start")
            row.prop(scene, "frame_end", text="⏪ End")
            row = box.row()
            row.operator("fusioncam.real_time_preview", text="👁️ Real-Time Preview")
            row.operator("fusioncam.switch_camera", text="🔄 Switch Camera")


            # ⚙️ PRO CAMERA CONTROLS
            box.label(text="Camera Pro Tools", icon="CAMERA_DATA")

            row = box.row()
            row.operator("camera.fit_to_frame", icon="CAMERA_DATA")
            row.operator("fusioncam.lock_camera", text="🔒 Lock Camera & Shading")
            row = box.row()
            row.operator("fusioncam.add_shake", text="📽️ Add Camera Shake")
            box.prop(camera.data, "dof.focus_distance", text="🎯 Focus Distance")
            row.operator("fusioncam.set_timeline_to_100", text="🔁 Set Timeline To 100")
            row = box.row()
            row.operator("fusioncam.bind_camera_to_object", text="📌 Bind Cinematic Camera")
            row.operator("fusioncam.bind_camera", text="📌 Bind Camera to Marker")
            box.enabled = is_pro_unlocked


            enabled = (camera is not None) and is_pro_unlocked
            box = layout.box()
            box.label(text="🎯 Depth of Field")
            box.enabled = (camera is not None)  # Show only if camera exists


            # Clipping
            # Perspective Type
            row = box.row()
            row.enabled = enabled
            row.prop(camera.data, "type", text="Type")


            is_pro = context.scene.fusioncam_pro_unlocked

            # 🔒 Focus Object
            row = box.row()
            row.enabled = is_pro
            row.prop(camera.data.dof, "focus_object", text="Focus on Object", icon='VIEWZOOM')
            # 🔒 Camera Name
            row = box.row()
            row.enabled = is_pro
            row.prop(camera, "name", text="Camera Name")

            row = box.row()
            row.enabled = is_pro
            row.prop(camera.data, "shift_x", text="Horizontal Shift")
            row.prop(camera.data, "shift_y", text="Vertical Shift")
            # Focal Length
            row = box.row()
            row.enabled = enabled
            row.prop(camera.data, "lens", text="📷 Focal Length")
            row.prop(camera.data, "show_safe_areas", text="Safe Area Guides", icon='OVERLAY')

            row = box.row()
            row.enabled = enabled
            row.prop(camera.data, "clip_start", text="Clip Start")
            row.prop(camera.data, "clip_end", text="Clip End")





            # DoF Toggle
            row = box.row()
            row.enabled = enabled
            row.prop(camera.data.dof, "use_dof", text="Enable DoF")


            # If DOF enabled
            if camera.data.dof.use_dof:
                row = box.row()
                row.enabled = enabled
                row.prop(camera.data.dof, "focus_distance", text="Focus Distance")

                row = box.row()
                row.enabled = enabled
                row.prop(camera.data.dof, "aperture_fstop", text="Aperture")

            # Show in Front
            row = box.row()
            row.enabled = enabled
            row.prop(camera.data, "show_in_front", text="Show in Front")

            # Show Limits
            row.enabled = enabled
            row.prop(camera.data, "show_limits", text="Show Limits")

            # Passepartout
            row = box.row()
            row.enabled = enabled
            row.prop(camera.data, "show_passepartout", text="Show Passepartout")

            if camera.data.show_passepartout:
                row = box.row()
                row.enabled = enabled
                row.prop(camera.data, "passepartout_alpha", text="Passepartout Alpha")

            # Composition Guides
            row = box.row()
            row.enabled = enabled
            row.prop(camera.data, "show_guide", text="Show Composition Guides")

            # Make Unselectable
            row = box.row()
            row.enabled = enabled
            row.prop(camera, "hide_select", text="Make Unselectable Camera")



#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# Operators                                                                                                                                                                           #
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# This section contains the operators that handle the functionality of the buttons and other UI elements.
# OPERATOR (Step 3)
class SelectiveObjectVisibilityProperties(bpy.types.PropertyGroup):
    enable_selective_visibility: bpy.props.BoolProperty(
        name="Enable Selective Visibility",
        description="Hide and disable all objects except the selected one",
        default=False
    )


# 🎯 Operator to Toggle DOF
class CAMERA_OT_ToggleDOF(bpy.types.Operator):
    """Toggle Depth of Field for the active camera"""
    bl_idname = "camera.toggle_dof"
    bl_label = "Toggle Camera DOF"

    enable_dof: bpy.props.BoolProperty()

    def execute(self, context):
        toggle_dof(self.enable_dof)
        return {'FINISHED'}


class OBJECT_OT_apply_keyframe_offset(bpy.types.Operator):
    bl_idname = "object.apply_keyframe_offset"
    bl_label = "Apply Keyframe Offset"
    bl_description = "Offsets the keyframes of the selected object by a specified amount"

    def execute(self, context):
        obj = context.object
        if not obj:
            self.report({'ERROR'}, "No object selected!")
            return {'CANCELLED'}  # ✅ Correct return value

        offset_value = context.scene.keyframe_offset_amount
        print(f"DEBUG: Offset Value = {offset_value}")

        # Function ko call karne ke baad ensure karo ke kuch return ho raha hai
        success = apply_keyframe_offset(obj, offset_value)

        if success:
            self.report({'INFO'}, f"Keyframes moved forward by {offset_value} frames")
            return {'FINISHED'}  # ✅ Blender ke operators ke liye zaroori hai
        else:
            self.report({'ERROR'}, "Keyframe offset failed!")
            return {'CANCELLED'}  # ✅ Error handling


class FUSIONCAM_OT_ApplyEasing(bpy.types.Operator):
    bl_idname = "fusioncam.apply_easing"
    bl_label = "Apply Easing"

    def execute(self, context):
        scene = context.scene
        easing_type = scene.Easing_Pro
        intensity = scene.Easing_Pro_intensity

        # Apply easing to all objects with keyframes
        for obj in bpy.context.scene.objects:
            if obj.animation_data:
                apply_easing_to_all_keyframes(obj, easing_type, intensity)

        self.report({'INFO'}, f"Applied {easing_type} easing to all keyframes!")
        return {'FINISHED'}


class ApplyEasingOperator(bpy.types.Operator):
    bl_idname = "object.apply_selected_easing"
    bl_label = "Apply Easing to Selected"

    def execute(self, context):
        apply_easing_to_selected_keyframes()
        return {'FINISHED'}

class FIT_PT_CameraPanel(bpy.types.Panel):
    """Creates a Panel in the UI"""
    bl_label = "Camera Tools"
    bl_idname = "FIT_PT_CameraPanel"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Camera"

    def draw(self, context):
        layout = self.layout
        layout.operator("camera.fit_to_frame", icon="CAMERA_DATA")

class FUSIONCAM_OT_BindCameraToMarker(bpy.types.Operator):
    """Bind Camera to a Timeline Marker"""
    bl_idname = "fusioncam.bind_camera"
    bl_label = "Bind Camera to Marker"

    frame: bpy.props.IntProperty(name="Frame")

    def __init__(self):
        self.frame = bpy.context.scene.frame_current  # Default to current frame

    def execute(self, context):
        scene = context.scene
        camera = scene.camera  # Active Camera

        if not camera:
            self.report({'WARNING'}, "No active camera found!")
            return {'CANCELLED'}

        # Get or create marker at the current frame
        marker = scene.timeline_markers.get(f"Marker_{self.frame}")
        if not marker:
            marker = scene.timeline_markers.new(name=f"Marker_{self.frame}", frame=self.frame)

        marker.camera = camera
        self.report({'INFO'}, f"Camera bound to frame {self.frame}")
        return {'FINISHED'}

class FUSIONCAM_OT_BindCameraToObject(bpy.types.Operator):
    bl_idname = "fusioncam.bind_camera_to_object"
    bl_label = "Bind Camera to Object"
    bl_description = "Bind the selected camera to a target object using a Track To constraint"

    def execute(self, context):
        # Get the active camera
        camera = context.scene.camera
        if not camera:
            self.report({'ERROR'}, "No active camera found!")
            return {'CANCELLED'}

        # Get the selected object (target)
        target_object = context.view_layer.objects.active
        if not target_object:
            self.report({'ERROR'}, "No target object selected!")
            return {'CANCELLED'}

        # Bind the camera to the target object
        bind_camera_to_object(camera, target_object)
        self.report({'INFO'}, f"Camera '{camera.name}' bound to '{target_object.name}'!")
        return {'FINISHED'}

class FUSIONCAM_OT_ApplyCamera(bpy.types.Operator):
    bl_idname = "fusioncam.apply_camera"
    bl_label = "Apply Camera Animation Preset"
    preset_type: bpy.props.StringProperty(default="FREE")

    def execute(self, context):
        if self.preset_type == "FREE":
            apply_camera_animation(context.scene.fusioncam_preset_free)
        else:
            if context.scene.fusioncam_pro_unlocked:
                apply_camera_animation(context.scene.fusioncam_preset)
            else:
                self.report({'ERROR'}, "Pro version is locked! Enter the key to unlock.")
        return {'FINISHED'}


class FUSIONCAM_OT_AddShake(bpy.types.Operator):
    bl_idname = "fusioncam.add_shake"
    bl_label = "Add Camera Shake"
    def execute(self, context):
        camera = bpy.context.scene.camera
        if camera:
            add_camera_shake(camera)
            self.report({'INFO'}, "Camera Shake added!")
        else:
            self.report({'ERROR'}, "No camera found!")
        return {'FINISHED'}

class FUSIONCAM_OT_AnimatePath(bpy.types.Operator):
    bl_idname = "fusioncam.animate_path"
    bl_label = "Animate Along Path"
    def execute(self, context):
        camera = bpy.context.scene.camera
        curve = bpy.context.scene.objects.get("Path")  # Assuming "Path" is the curve object
        if camera and curve:
            animate_camera_along_path(camera, curve)
            self.report({'INFO'}, "Animation along path applied!")
        else:
            self.report({'ERROR'}, "Camera or path not found!")
        return {'FINISHED'}

class FUSIONCAM_OT_RealTimePreview(bpy.types.Operator):
    bl_idname = "fusioncam.real_time_preview"
    bl_label = "Real-Time Preview"
    def execute(self, context):
        camera = bpy.context.scene.camera
        if camera:
            real_time_preview(camera)
            self.report({'INFO'}, "Real-Time Preview activated!")
        else:
            self.report({'ERROR'}, "No camera found!")
        return {'FINISHED'}

class FUSIONCAM_OT_AICamera(bpy.types.Operator):
    bl_idname = "fusioncam.ai_camera"
    bl_label = "AI-Powered Camera"

    def execute(self, context):
        camera = bpy.context.scene.camera
        target_obj = next((obj for obj in bpy.context.scene.objects if "Product" in obj.name), None)

        if camera and target_obj:
            ai_suggest_camera_angle(camera, target_obj)
            self.report({'INFO'}, "AI camera angle set!")
        else:
            self.report({'ERROR'}, "Please Add Fusion Cam & Rename Your Object 'Product'!")
        return {'FINISHED'}



class FUSIONCAM_OT_LockCamera(bpy.types.Operator):
    bl_idname = "fusioncam.lock_camera"
    bl_label = "Lock Camera to Viewport"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        camera = bpy.context.scene.camera
        if not camera:
            self.report({'ERROR'}, "No camera found!")
            return {'CANCELLED'}

        # Check if camera is already locked to view
        if context.space_data.region_3d.view_perspective == 'CAMERA' and context.space_data.region_3d.is_perspective:
            self.report({'INFO'}, "Camera is already locked to viewport!")
            return {'FINISHED'}

        # Lock camera to viewport
        bpy.ops.view3d.camera_to_view()

        # Enable shading mode
        for area in bpy.context.screen.areas:
            if area.type == 'VIEW_3D':
                for space in area.spaces:
                    if space.type == 'VIEW_3D':
                        space.shading.type = 'MATERIAL'  # Or 'MATERIAL'/'SOLID'

        self.report({'INFO'}, "Camera locked to viewport and shading mode enabled!")
        return {'FINISHED'}

class FUSIONCAM_OT_ApplyCamera(bpy.types.Operator):
    bl_idname = "fusioncam.apply_camera"
    bl_label = "Apply Camera Animation Preset"
    preset_type: bpy.props.StringProperty(default="FREE")  # FREE or PRO

    def execute(self, context):
        if self.preset_type == "FREE":
            apply_camera_animation(context.scene.fusioncam_preset_free)
        else:
            if context.scene.fusioncam_pro_unlocked:
                apply_camera_animation(context.scene.fusioncam_preset)
            else:
                self.report({'ERROR'}, "Pro version is locked! Enter the key to unlock.")
        return {'FINISHED'}

class FUSIONCAM_OT_ValidateKey(bpy.types.Operator):
    bl_idname = "fusioncam.validate_key"
    bl_label = "🔐 Validate FusionCam Key"

    def execute(self, context):
        scene = context.scene
        key = scene.fusioncam_pro_key

        if not key:
            self.report({'ERROR'}, "Please enter a license key.")
            return {'CANCELLED'}

        data = {
            "product": "fusion_cam_pro",
            "license_key": key
        }

        try:
            import urllib.request
            import json
            import os

            req = urllib.request.Request(
                url="https://fusionart.net/FusionHubAddon/FusionLicenseKeys/Verify_Pro_Key_All_Products.php",
                data=json.dumps(data).encode(),
                headers={'Content-Type': 'application/json'}
            )

            response = urllib.request.urlopen(req)
            result = json.loads(response.read().decode())

            # 🔍 DEBUG LINE (yeh print karega server ka response in Blender Console)
            print("💬 SERVER RESPONSE:", result)

            if result.get("status") == "success":
                with open(os.path.join(bpy.utils.user_resource('CONFIG'), "fusioncam_key.json"), "w") as f:
                    json.dump({"key": key}, f)

                scene.fusioncam_pro_unlocked = True
                self.report({'INFO'}, "FusionCam Pro unlocked successfully!")
                bpy.context.area.tag_redraw()

            else:
                scene.fusioncam_pro_unlocked = False
                self.report({'ERROR'}, "Invalid or expired key.")

        except Exception as e:
            self.report({'ERROR'}, f"Key verification failed: {e}")
            return {'CANCELLED'}

        return {'FINISHED'}


class FUSIONCAM_OT_AddCinematicCamera(bpy.types.Operator):
    bl_idname = "fusioncam.add_cinematic_camera"
    bl_label = "Add Cinematic Camera"

    def execute(self, context):
        preset = context.scene.fusioncam_cinematic_preset
        if preset:
            add_cinematic_camera(preset)
            self.report({'INFO'}, f"Added {preset} cinematic camera to scene.")
        else:
            self.report({'ERROR'}, "No preset selected.")
        return {'FINISHED'}

class FUSIONCAM_OT_NextPreset(bpy.types.Operator):
    bl_idname = "fusioncam.next_preset"
    bl_label = "Next Preset"

    def execute(self, context):
        if context.scene.fusioncam_tabs == "FREE":
            presets = FREE_PRESETS
            current_preset = context.scene.fusioncam_preset_free
        else:
            presets = FREE_PRESETS + PRO_PRESETS
            current_preset = context.scene.fusioncam_preset

        # Find the current index
        current_index = next((i for i, (id, name, desc) in enumerate(presets) if id == current_preset), 0)
        # Move to the next preset
        next_index = (current_index + 1) % len(presets)
        next_preset = presets[next_index][0]

        # Update the selected preset
        if context.scene.fusioncam_tabs == "FREE":
            context.scene.fusioncam_preset_free = next_preset
        else:
            context.scene.fusioncam_preset = next_preset

        # Apply the new preset
        apply_camera_animation(next_preset)

        return {'FINISHED'}

class FUSIONCAM_OT_PreviousPreset(bpy.types.Operator):
    bl_idname = "fusioncam.previous_preset"
    bl_label = "Previous Preset"

    def execute(self, context):
        if context.scene.fusioncam_tabs == "FREE":
            presets = FREE_PRESETS
            current_preset = context.scene.fusioncam_preset_free
        else:
            presets = FREE_PRESETS + PRO_PRESETS
            current_preset = context.scene.fusioncam_preset

        # Find the current index
        current_index = next((i for i, (id, name, desc) in enumerate(presets) if id == current_preset), 0)

        # Move to the previous preset
        previous_index = (current_index - 1) % len(presets)
        previous_preset = presets[previous_index][0]

        # Update the selected preset
        if context.scene.fusioncam_tabs == "FREE":
            context.scene.fusioncam_preset_free = previous_preset
        else:
            context.scene.fusioncam_preset = previous_preset

        # Apply the new preset
        apply_camera_animation(previous_preset)

        return {'FINISHED'}

class FUSIONCAM_OT_SwitchCamera(bpy.types.Operator):
    bl_idname = "fusioncam.switch_camera"
    bl_label = "Switch Camera"

    def execute(self, context):
        # Get all cameras in the scene
        cameras = [obj for obj in bpy.context.scene.objects if obj.type == 'CAMERA']

        if not cameras:
            self.report({'ERROR'}, "No cameras found in the scene!")
            return {'CANCELLED'}

        # Find the current active camera
        active_camera = bpy.context.scene.camera

        if not active_camera:
            # If no active camera, set the first camera as active
            bpy.context.scene.camera = cameras[0]
            self.report({'INFO'}, f"Switched to camera: {cameras[0].name}")
            return {'FINISHED'}

        # Find the index of the active camera
        current_index = cameras.index(active_camera) if active_camera in cameras else -1

        # Move to the next camera
        next_index = (current_index + 1) % len(cameras)
        next_camera = cameras[next_index]

        # Set the new active camera
        bpy.context.scene.camera = next_camera
        self.report({'INFO'}, f"Switched to camera: {next_camera.name}")

        return {'FINISHED'}

class FUSIONCAM_OT_Settings(bpy.types.Operator):
    bl_idname = "fusioncam.apply_settings"
    bl_label = "Apply Settings"

    def execute(self, context):
        cam = bpy.context.scene.camera
        if cam:
            apply_settings(cam)  # Ye function defined hona chahiye
        return {'FINISHED'}

#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
class FUSIONCAM_OT_AddShake(bpy.types.Operator):
    bl_idname = "fusioncam.add_shake"
    bl_label = "Add Camera Shake"
    def execute(self, context):
        camera = bpy.context.scene.camera
        if camera:
            add_camera_shake(camera)
            self.report({'INFO'}, "Camera Shake added!")
        else:
            self.report({'ERROR'}, "No camera found!")
        return {'FINISHED'}

    # Operators for New Pro Features

class FUSIONCAM_OT_LoopAnimation(bpy.types.Operator):
    bl_idname = "fusioncam.set_timeline_to_100"
    bl_label = "Set_Timeline_To_100"

    def execute(self, context):
        camera = bpy.context.scene.camera
        if camera:
            set_timeline_to_100(camera, 1, 100)  # Example frames
            self.report({'INFO'}, "Animation loop set!")
        else:
            self.report({'ERROR'}, "No camera found!")
        return {'FINISHED'}


#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# Registration                                                                                                                                                                        #
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# This section registers and unregisters the add-on in Blender.
classes = [
    SelectiveObjectVisibilityProperties,

]


# Registration
def register():
    for cls in classes:
        bpy.utils.register_class(cls)
    bpy.types.Scene.selective_object_visibility = bpy.props.PointerProperty(type=SelectiveObjectVisibilityProperties)
    bpy.app.handlers.depsgraph_update_post.append(update_object_visibility)

    # Update when selection changes or property changes
    bpy.app.handlers.depsgraph_update_post.append(update_object_visibility)
    bpy.app.handlers.frame_change_post.append(update_object_visibility)

    bpy.types.Scene.fusioncam_selective_visibility = bpy.props.EnumProperty(
        name="Selective Visibility",
        items=[
            ('OFF', 'OFF', 'Turn off selective visibility'),
            ('ON', 'ON', 'Enable selective visibility')
        ],
        default='OFF'
    )
    bpy.types.Scene.fusioncam_pro_key = bpy.props.StringProperty(name="FusionCam Pro Key")

    bpy.types.Scene.fusioncam_pro_unlocked = bpy.props.BoolProperty(...)
    bpy.utils.register_class(FUSIONCAM_OT_ValidateKey)

    bpy.utils.register_class(FUSIONCAM_OT_Settings)
    bpy.utils.register_class(FUSIONCAM_OT_BindCameraToObject)
    bpy.utils.register_class(FUSIONCAM_PT_Panel)
    bpy.utils.register_class(FUSIONCAM_OT_ApplyCamera)
    bpy.utils.register_class(FUSIONCAM_OT_AddCinematicCamera)
    bpy.utils.register_class(FUSIONCAM_OT_NextPreset)
    bpy.utils.register_class(FUSIONCAM_OT_PreviousPreset)
    bpy.utils.register_class(FUSIONCAM_OT_SwitchCamera)
    bpy.utils.register_class(FUSIONCAM_OT_LockCamera)
    bpy.utils.register_class(FUSIONCAM_OT_RealTimePreview)
    bpy.utils.register_class(FUSIONCAM_OT_AICamera)
    bpy.utils.register_class(FUSIONCAM_OT_AddShake)
    bpy.utils.register_class(FUSIONCAM_OT_LoopAnimation)
    bpy.utils.register_class(FUSIONCAM_OT_BindCameraToMarker)
    bpy.utils.register_class(FIT_OT_CameraToProduct)
    bpy.utils.register_class(ApplyEasingOperator)
    bpy.utils.register_class(FUSIONCAM_OT_ApplyEasing)
    bpy.utils.register_class(OBJECT_OT_apply_keyframe_offset)
    bpy.types.Scene.fusioncam_density = bpy.props.FloatProperty(name="FusionCam Density", default=1.0)

    bpy.types.Scene.fusioncam_license_key = bpy.props.StringProperty(name="License Key")
    bpy.types.Scene.fusioncam_pro_unlocked = bpy.props.BoolProperty(name="FusionCam Unlocked", default=False)

    bpy.utils.register_class(FUSIONCAM_OT_LockProVersion)




    # REGISTER PROPERTY (Step 1)
    bpy.types.Scene.keyframe_offset_amount = bpy.props.IntProperty(
        name="Offset Amount",
        description="Frames to offset between animation layers",
        default=10,  # Default value 10
        min=1  # Minimum 1 frame
    )




    bpy.types.Scene.fusioncam_preset_free = bpy.props.EnumProperty(
        name="Preset",
        items=FREE_PRESETS
    )

    bpy.types.Scene.fusioncam_preset = bpy.props.EnumProperty(
        name="Preset",
        items=FREE_PRESETS + PRO_PRESETS
    )

    bpy.types.Scene.Easing_Pro = bpy.props.EnumProperty(
        name="Easing Type",
        description="Select the type of easing",
        items=[
            ('LINEAR', "LINEAR", "No easing"),
            ('EASE_IN', "Ease In", "Slow start"),
            ('EASE_OUT', "Ease Out", "Slow end"),
            ('EASE_IN_OUT', "Ease In-Out", "Slow start and end"),
            ('BEZIER', "Bezier", "Smooth Bezier interpolation"),
            ('EXPONENTIAL', "Exponential", "Exponential interpolation"),
            ('BOUNCE', "Bounce", "Bounce interpolation"),
            ('ELASTIC', "Elastic", "Elastic interpolation"),
            ('SINE', "Sine", "Sine interpolation"),
            ('QUADRATIC', "Quadratic", "Quadratic interpolation"),
            ('CUBIC', "Cubic", "Cubic interpolation"),
            ('QUARTIC', "Quartic", "Quartic interpolation"),
            ('QUINTIC', "Quintic", "Quintic interpolation"),
            ('AUTO_SMOOTH', "Auto Smooth", "Automatic smooth interpolation"),
            ('AUTO_LINEAR', "Auto LINEAR", "Automatic LINEAR interpolation"),
        ],
        default='LINEAR'
    )
    bpy.types.Scene.fusioncam_random = bpy.props.BoolProperty(name="LOCKCAMERA_OT_Enable", default=False)
    bpy.types.Scene.fusioncam_pro_key = bpy.props.StringProperty(name="Pro Key", default="")
    bpy.types.Scene.fusioncam_pro_unlocked = bpy.props.BoolProperty(name="Pro Unlocked", default=False)
    bpy.types.Scene.fusioncam_tabs = bpy.props.EnumProperty(
        name="Tabs",
        items=[
            ("FREE", "Free", "Free Version"),
            ("PRO", "Pro", "Pro Version")
        ],
        default="FREE"
    )
    bpy.types.Scene.fusioncam_cinematic_preset = bpy.props.EnumProperty(
        name="Cinematic Preset",
        items=CINEMATIC_PRESETS
    )

    # Property ko register karna zaroori hai
    bpy.types.Scene.Easing_Pro_intensity = bpy.props.FloatProperty(
        name="Easing Intensity",
        description="Controls the easing effect intensity",
        default=1.0,
        min=0.1,
        max=10.0
    )
    bpy.utils.register_class(CAMERA_OT_ToggleDOF)
    bpy.types.Scene.fusioncam_dof_toggle = bpy.props.BoolProperty(
        name="Enable DOF",
        description="Toggle DOF",
        default=False,
        update=lambda s, c: toggle_dof(s.fusioncam_dof_toggle),
    )

    bpy.types.Scene.fusioncam_show_camera_warning = bpy.props.BoolProperty(default=True)
    bpy.app.handlers.depsgraph_update_post.append(check_camera_presence)


    bpy.types.Scene.fusioncam_dynamic_tip = bpy.props.StringProperty(default="")
    bpy.app.handlers.depsgraph_update_post.append(update_dynamic_pro_tip)

def apply_saved_key_safely():
    saved_key = load_fusioncam_key()
    if saved_key:
        try:
            bpy.context.scene.fusioncam_pro_key = saved_key  # 🔑 Also set the key field
            bpy.context.scene.fusioncam_pro_unlocked = True  # ✅ Unlock UI
            print("[FusionCam] Key loaded and Pro version unlocked.")
        except Exception as e:
            print(f"[FusionCam] Safe Key Error: {e}")
    return None  # Must return None so it only runs once

def unregister():
    bpy.utils.unregister_class(FUSIONCAM_OT_Settings)
    bpy.utils.unregister_class(FUSIONCAM_OT_BindCameraToObject)
    bpy.utils.unregister_class(FUSIONCAM_PT_Panel)
    bpy.utils.unregister_class(FUSIONCAM_OT_ApplyCamera)
    bpy.utils.unregister_class(FUSIONCAM_OT_AddCinematicCamera)
    bpy.utils.unregister_class(FUSIONCAM_OT_NextPreset)
    bpy.utils.unregister_class(FUSIONCAM_OT_PreviousPreset)
    bpy.utils.unregister_class(FUSIONCAM_OT_SwitchCamera)
    bpy.utils.unregister_class(FUSIONCAM_OT_LockCamera)
    bpy.utils.unregister_class(FUSIONCAM_OT_RealTimePreview)
    bpy.utils.unregister_class(FUSIONCAM_OT_AICamera)
    bpy.utils.unregister_class(FUSIONCAM_OT_AddShake)
    bpy.utils.unregister_class(FUSIONCAM_OT_LoopAnimation)
    bpy.utils.unregister_class(FUSIONCAM_OT_BindCameraToMarker)
    bpy.utils.unregister_class(FIT_OT_CameraToProduct)
    bpy.utils.unregister_class(ApplyEasingOperator)
    bpy.utils.unregister_class(OBJECT_OT_apply_keyframe_offset)

    bpy.utils.unregister_class(CAMERA_OT_ToggleDOF)
    del bpy.types.Scene.fusioncam_dof_toggle

    bpy.app.handlers.frame_change_post.remove(update_object_visibility)
    bpy.app.handlers.depsgraph_update_post.remove(update_object_visibility)
    del bpy.types.Scene.selective_object_visibility
    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)
    bpy.app.handlers.depsgraph_update_post.remove(check_camera_presence)
    del bpy.types.Scene.fusioncam_show_camera_warning


    bpy.app.handlers.depsgraph_update_post.remove(update_dynamic_pro_tip)
    del bpy.types.Scene.fusioncam_dynamic_tip


    # Property ko unregister bhi karna hoga warna error ayega
    del bpy.types.Scene.keyframe_offset_amount
    del bpy.types.Scene.Easing_Pro_intensity
    del bpy.types.Scene.fusioncam_preset_free
    del bpy.types.Scene.fusioncam_preset
    del bpy.types.Scene.Easing_Pro
    del bpy.types.Scene.fusioncam_density
    del bpy.types.Scene.fusioncam_random
    del bpy.types.Scene.fusioncam_pro_key
    del bpy.types.Scene.fusioncam_tabs
    del bpy.types.Scene.fusioncam_cinematic_preset

if hasattr(bpy.types.Scene, "fusioncam_density"):
    del bpy.types.Scene.fusioncam_density

#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# Main Execution                                                                                                                                                                      #
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# This is the entry point of the script. It registers the add-on when the script is executed.

if __name__ == "__main__":
    register()